@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -13,9 +13,9 @@ use warnings;
 
 our @ObjectDependencies = (
     'Kernel::Config',
+    'Kernel::System::DateTime',
     'Kernel::System::Log',
     'Kernel::System::Main',
-    'Kernel::System::Time',
 );
 
 sub new {
@@ -55,6 +55,17 @@ sub LoadPreferences {
     # our results: "PostgreSQL 9.2.4", "PostgreSQL 9.1.9".
     $Self->{'DB::Version'} = "SELECT SUBSTRING(VERSION(), 'PostgreSQL [0-9\.]*')";
 
+    $Self->{'DB::ListTables'} = <<'EOF';
+SELECT
+    table_name
+FROM
+    information_schema.tables
+WHERE
+    table_type = 'BASE TABLE'
+    AND table_schema NOT IN ('pg_catalog', 'information_schema')
+ORDER BY table_name
+EOF
+
     # dbi attributes
     $Self->{'DB::Attribute'} = {};
 
@@ -70,7 +81,7 @@ sub LoadPreferences {
     $Self->{'DB::ShellConnect'} = 'SET standard_conforming_strings TO ON';
 
     # init sql setting on db connect
-    $Self->{'DB::Connect'} = "SET standard_conforming_strings TO ON;\n SET NAMES 'utf8';";
+    $Self->{'DB::Connect'} = "SET standard_conforming_strings TO ON;\n SET datestyle TO 'iso';\n SET NAMES 'utf8';";
     return 1;
 }
 
@@ -154,6 +165,7 @@ sub TableCreate {
     my %Uniq         = ();
     my $PrimaryKey   = '';
     my @Return       = ();
+
     for my $Tag (@Param) {
 
         if (
@@ -279,31 +291,27 @@ sub TableCreate {
     $SQL .= "\n";
     push @Return, $SQLStart . $SQL . $SQLEnd;
 
-    # add indexs
+    # add indexes
     for my $Name ( sort keys %Index ) {
-        push(
-            @Return,
+        push @Return,
             $Self->IndexCreate(
-                TableName => $TableName,
-                Name      => $Name,
-                Data      => $Index{$Name},
-            ),
-        );
+            TableName => $TableName,
+            Name      => $Name,
+            Data      => $Index{$Name},
+            );
     }
 
     # add foreign keys
     for my $ForeignKey ( sort keys %Foreign ) {
         my @Array = @{ $Foreign{$ForeignKey} };
         for ( 0 .. $#Array ) {
-            push(
-                @{ $Self->{Post} },
+            push @{ $Self->{Post} },
                 $Self->ForeignKeyCreate(
-                    LocalTableName   => $TableName,
-                    Local            => $Array[$_]->{Local},
-                    ForeignTableName => $ForeignKey,
-                    Foreign          => $Array[$_]->{Foreign},
-                ),
-            );
+                LocalTableName   => $TableName,
+                Local            => $Array[$_]->{Local},
+                ForeignTableName => $ForeignKey,
+                Foreign          => $Array[$_]->{Foreign},
+                );
         }
     }
     return @Return;
@@ -347,6 +355,10 @@ sub TableAlter {
     my @Reference     = ();
     my $Table         = '';
 
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
     TAG:
     for my $Tag (@Param) {
 
@@ -362,7 +374,36 @@ sub TableAlter {
 
             # rename table
             if ( $Tag->{NameOld} && $Tag->{NameNew} ) {
+
+                # PostgreSQL uses sequences for primary key value generation. These are global
+                #   entities and not renamed when tables are renamed. Rename them also to keep
+                #   them consistent with new systems.
                 push @SQL, $SQLStart . "ALTER TABLE $Tag->{NameOld} RENAME TO $Tag->{NameNew}";
+
+                my $OldSequence = $Self->_SequenceName(
+                    TableName => $Tag->{NameOld},
+                );
+
+                my $NewSequence = $Self->_SequenceName(
+                    TableName => $Tag->{NameNew},
+                );
+
+                # Build SQL to rename sequence (only if a sequence exists).
+                my $RenameSequenceSQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF EXISTS (
+    SELECT 1
+    FROM pg_class
+    WHERE relkind = 'S' and relname = '$OldSequence'
+    ) THEN
+    ALTER SEQUENCE $OldSequence RENAME TO $NewSequence;
+    END IF;
+END$DollarDollar;
+EOF
+
+                push @SQL, $SQLStart . $RenameSequenceSQL;
+
             }
             $SQLStart .= "ALTER TABLE $Table";
         }
@@ -428,8 +469,8 @@ sub TableAlter {
             # if there is an AutoIncrement column no other changes are needed
             next TAG if $Tag->{AutoIncrement} && $Tag->{AutoIncrement} =~ /^true$/i;
 
-            # remove possible default
-            push @SQL, "ALTER TABLE $Table ALTER $Tag->{NameNew} DROP DEFAULT";
+            # set default as null
+            push @SQL, "ALTER TABLE $Table ALTER $Tag->{NameNew} DROP NOT NULL";
 
             # investigate the default value
             my $Default = '';
@@ -521,23 +562,37 @@ sub IndexCreate {
             return;
         }
     }
-    my $SQL   = "CREATE INDEX $Param{Name} ON $Param{TableName} (";
-    my @Array = @{ $Param{Data} };
+    my $CreateIndexSQL = "CREATE INDEX $Param{Name} ON $Param{TableName} (";
+    my @Array          = @{ $Param{Data} };
     for ( 0 .. $#Array ) {
         if ( $_ > 0 ) {
-            $SQL .= ', ';
-        }
-        $SQL .= $Array[$_]->{Name};
-        if ( $Array[$_]->{Size} ) {
-
-            #           $SQL .= "($Array[$_]->{Size})";
+            $CreateIndexSQL .= ', ';
         }
+        $CreateIndexSQL .= $Array[$_]->{Name};
     }
-    $SQL .= ')';
+    $CreateIndexSQL .= ')';
+
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
+    # build SQL to create index within a "try/catch block"
+    # to prevent errors if index exists already
+    $CreateIndexSQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF NOT EXISTS (
+    SELECT 1
+    FROM pg_indexes
+    WHERE LOWER(indexname) = LOWER('$Param{Name}')
+    ) THEN
+    $CreateIndexSQL;
+END IF;
+END$DollarDollar;
+EOF
 
     # return SQL
-    return ($SQL);
-
+    return ($CreateIndexSQL);
 }
 
 sub IndexDrop {
@@ -548,13 +603,35 @@ sub IndexDrop {
         if ( !$Param{$_} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message  => "Need $_!"
+                Message  => "Need $_!",
             );
             return;
         }
     }
-    my $SQL = 'DROP INDEX ' . $Param{Name};
-    return ($SQL);
+    my $DropIndexSQL = 'DROP INDEX ' . $Param{Name};
+
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
+    # build SQL to drop index within a "try/catch block"
+    # to prevent errors if index does not exist
+    $DropIndexSQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF EXISTS (
+    SELECT 1
+    FROM pg_indexes
+    WHERE LOWER(indexname) = LOWER('$Param{Name}')
+    ) THEN
+    $DropIndexSQL;
+END IF;
+END$DollarDollar;
+EOF
+
+    # return SQL
+    return ($DropIndexSQL);
+
 }
 
 sub ForeignKeyCreate {
@@ -565,7 +642,7 @@ sub ForeignKeyCreate {
         if ( !$Param{$_} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message  => "Need $_!"
+                Message  => "Need $_!",
             );
             return;
         }
@@ -583,10 +660,30 @@ sub ForeignKeyCreate {
     }
 
     # add foreign key
-    my $SQL = "ALTER TABLE $Param{LocalTableName} ADD CONSTRAINT $ForeignKey FOREIGN KEY "
+    my $CreateForeignKeySQL = "ALTER TABLE $Param{LocalTableName} ADD CONSTRAINT $ForeignKey FOREIGN KEY "
         . "($Param{Local}) REFERENCES $Param{ForeignTableName} ($Param{Foreign})";
 
-    return ($SQL);
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
+    # build SQL to create foreign key within a "try/catch block"
+    # to prevent errors if foreign key exists already
+    $CreateForeignKeySQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF NOT EXISTS (
+    SELECT 1
+    FROM pg_constraint
+    WHERE LOWER(conname) = LOWER('$ForeignKey')
+    ) THEN
+    $CreateForeignKeySQL;
+END IF;
+END$DollarDollar;
+EOF
+
+    # return SQL
+    return ($CreateForeignKeySQL);
 }
 
 sub ForeignKeyDrop {
@@ -615,9 +712,29 @@ sub ForeignKeyDrop {
     }
 
     # drop foreign key
-    my $SQL = "ALTER TABLE $Param{LocalTableName} DROP CONSTRAINT $ForeignKey";
+    my $DropForeignKeySQL = "ALTER TABLE $Param{LocalTableName} DROP CONSTRAINT $ForeignKey";
 
-    return ($SQL);
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
+    # build SQL to drop foreign key within a "try/catch block"
+    # to prevent errors if foreign key does not exist
+    $DropForeignKeySQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF EXISTS (
+    SELECT 1
+    FROM pg_constraint
+    WHERE LOWER(conname) = LOWER('$ForeignKey')
+    ) THEN
+    $DropForeignKeySQL;
+END IF;
+END$DollarDollar;
+EOF
+
+    # return SQL
+    return ($DropForeignKeySQL);
 }
 
 sub UniqueCreate {
@@ -633,19 +750,37 @@ sub UniqueCreate {
             return;
         }
     }
-    my $SQL   = "ALTER TABLE $Param{TableName} ADD CONSTRAINT $Param{Name} UNIQUE (";
-    my @Array = @{ $Param{Data} };
+    my $CreateUniqueSQL = "ALTER TABLE $Param{TableName} ADD CONSTRAINT $Param{Name} UNIQUE (";
+    my @Array           = @{ $Param{Data} };
     for ( 0 .. $#Array ) {
         if ( $_ > 0 ) {
-            $SQL .= ', ';
+            $CreateUniqueSQL .= ', ';
         }
-        $SQL .= $Array[$_]->{Name};
+        $CreateUniqueSQL .= $Array[$_]->{Name};
     }
-    $SQL .= ')';
+    $CreateUniqueSQL .= ')';
+
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
+    # build SQL to create unique constraint within a "try/catch block"
+    # to prevent errors if unique constraint does already exist
+    $CreateUniqueSQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF NOT EXISTS (
+    SELECT 1
+    FROM pg_constraint
+    WHERE LOWER(conname) = LOWER('$Param{Name}')
+    ) THEN
+    $CreateUniqueSQL;
+END IF;
+END$DollarDollar;
+EOF
 
     # return SQL
-    return ($SQL);
-
+    return ($CreateUniqueSQL);
 }
 
 sub UniqueDrop {
@@ -661,16 +796,37 @@ sub UniqueDrop {
             return;
         }
     }
-    my $SQL = "ALTER TABLE $Param{TableName} DROP CONSTRAINT $Param{Name}";
-    return ($SQL);
+    my $DropUniqueSQL = "ALTER TABLE $Param{TableName} DROP CONSTRAINT $Param{Name}";
+
+    # put two literal dollar characters in a string
+    # this is needed for the postgres 'do' statement
+    my $DollarDollar = '$$';
+
+    # build SQL to drop unique constraint within a "try/catch block"
+    # to prevent errors if unique constraint does not exist
+    $DropUniqueSQL = <<"EOF";
+DO $DollarDollar
+BEGIN
+IF EXISTS (
+    SELECT 1
+    FROM pg_constraint
+    WHERE LOWER(conname) = LOWER('$Param{Name}')
+    ) THEN
+    $DropUniqueSQL;
+END IF;
+END$DollarDollar;
+EOF
+
+    # return SQL
+    return ($DropUniqueSQL);
 }
 
 sub Insert {
     my ( $Self, @Param ) = @_;
 
     # get needed objects
-    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
-    my $TimeObject   = $Kernel::OM->Get('Kernel::System::Time');
+    my $ConfigObject   = $Kernel::OM->Get('Kernel::Config');
+    my $DateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
 
     my $SQL    = '';
     my @Keys   = ();
@@ -738,7 +894,7 @@ sub Insert {
                 $Value .= $Tmp;
             }
             else {
-                my $Timestamp = $TimeObject->CurrentTimestamp();
+                my $Timestamp = $DateTimeObject->ToString();
                 $Value .= '\'' . $Timestamp . '\'';
             }
         }
@@ -774,4 +930,21 @@ sub _TypeTranslation {
     return $Tag;
 }
 
+sub _SequenceName {
+    my ( $Self, %Param ) = @_;
+
+    # check needed stuff
+    if ( !$Param{TableName} ) {
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'error',
+            Message  => "Need TableName!",
+        );
+        return;
+    }
+
+    my $Sequence = $Param{TableName} . '_id_seq';
+
+    return $Sequence;
+}
+
 1;
